﻿'  MaestroEasyExampleVb:
'    Simple example GUI for the Maestro USB Servo Controller, written in
'    Visual Basic .NET.
'    
'    Features:
'       Temporary native USB connection using Usc class.
'       Button for disabling channel 0.
'       Button for setting target of channel 0 to 1000 us.
'       Button for setting target of channel 0 to 2000 us.
' 
'  NOTE: Channel 0 should be configured as a servo channel for this program
'  to work.  You must also connect USB and servo power, and connect a servo
'  to channel 0.  If this program does not work, use the Maestro Control
'  Center to check what errors are occurring.

Imports Pololu.UsbWrapper
Imports Pololu.Usc

Imports Microsoft.DirectX
Imports Microsoft.DirectX.DirectInput

Imports System
Imports System.Text
Imports System.ComponentModel

Public Class MainWindow
    Inherits System.Windows.Forms.Form

    ''' <summary>
    ''' This subroutine runs when the user clicks the Target=1000us button.
    ''' </summary>
    Sub Button1000_Click(ByVal sender As Object, ByVal e As EventArgs) Handles Button1000.Click
        TrySetTarget(0, 1000 * 4) ' Set the target of channel 0 to 1000 microseconds.
    End Sub




    Public Shared state As New JoystickState
    Private applicationDevice As Device = Nothing
    Public Shared numPOVs As Integer = 0
    Private SliderCount As Integer = 0 ' number of returned slider controls
    Private WithEvents timer1 As New Timer
    Public n As Integer = 0
    Public servo(24) As Byte

    Private Function InitDirectInput() As Boolean
        For Each instance As DeviceInstance In Manager.GetDevices(DeviceClass.GameControl, EnumDevicesFlags.AttachedOnly)
            applicationDevice = New Device(instance.InstanceGuid)
            Exit For
        Next
        If applicationDevice Is Nothing Then
            MessageBox.Show("Unable to create a Joystick device. Sample will exit.", "No joystick found")
            Return False
        End If
        applicationDevice.SetDataFormat(DeviceDataFormat.Joystick)
        applicationDevice.SetCooperativeLevel(Me, CooperativeLevelFlags.Exclusive Or CooperativeLevelFlags.Foreground)
        For Each d As DeviceObjectInstance In applicationDevice.Objects
            If Not (d.ObjectId And Convert.ToInt32(DeviceObjectTypeFlags.Axis)) = 0 Then
                applicationDevice.Properties.SetRange(ParameterHow.ById, d.ObjectId, New InputRange(0, +255))
                UpdateControls(d)
            End If
        Next
        Return True
    End Function

    Private Sub buttonExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buttonExit.Click
        SerialPort1.Close()

        Close()
    End Sub

    Public Sub GetData()
        If applicationDevice Is Nothing Then Return
        Try
            applicationDevice.Poll()
        Catch notAcqEx As NotAcquiredException
            Try
                applicationDevice.Acquire()
            Catch iex As InputException
                Return
            End Try
        Catch inpLostEx As InputLostException
            Try
                applicationDevice.Acquire()
            Catch iex As InputException
                Return
            End Try
        End Try ' Get the state of the device.
        Try
            state = applicationDevice.CurrentJoystickState
        Catch iex As InputException
            Return
        End Try
        UpdateUI()
    End Sub

    Function IsButtonPressed(ByVal buttonstate As Byte) As Boolean
        Return ((buttonstate And &H80) = &H80)
    End Function

    Public Sub UpdateControls(ByVal d As DeviceObjectInstance)
        If (ObjectTypeGuid.XAxis.Equals(d.ObjectType)) Then
            labelXAxis.Enabled = True
            'labelXAxisText.Enabled = True
        End If
        If (ObjectTypeGuid.YAxis.Equals(d.ObjectType)) Then
            'labelYAxis.Enabled = True
            'labelYAxisText.Enabled = True
        End If
        If (ObjectTypeGuid.ZAxis.Equals(d.ObjectType)) Then
            'labelZAxis.Enabled = True
            'labelZAxisText.Enabled = True
        End If
        If (ObjectTypeGuid.RxAxis.Equals(d.ObjectType)) Then
            'labelXRotation.Enabled = True
            'labelXRotationText.Enabled = True
        End If
        If (ObjectTypeGuid.RyAxis.Equals(d.ObjectType)) Then
            'labelYRotation.Enabled = True
            'labelYRotationText.Enabled = True
        End If

        If (ObjectTypeGuid.RzAxis.Equals(d.ObjectType)) Then
            'labelZRotation.Enabled = True
            'labelZRotationText.Enabled = True
        End If
        If (ObjectTypeGuid.Slider.Equals(d.ObjectType)) Then
            SliderCount += 1
            'labelSlider0.Enabled = True
            'labelSlider0Text.Enabled = True
        End If
        If (ObjectTypeGuid.PointOfView.Equals(d.ObjectType)) Then
            numPOVs += 1
            'labelPOV0.Enabled = True
            'labelPOV0Text.Enabled = True
        End If
    End Sub

    Private Sub UpdateUI()
        Dim strText As String = ""
        labelXAxis.Text = state.X.ToString()
        'labelYAxis.Text = state.Y.ToString()
        'labelZAxis.Text = state.Z.ToString()
        'labelXRotation.Text = state.Rx.ToString()
        'labelYRotation.Text = state.Ry.ToString()
        'labelZRotation.Text = state.Rz.ToString()
        Dim slider As Integer() = state.GetSlider()
        'labelSlider0.Text = slider(0).ToString()
        Dim pov As Integer() = state.GetPointOfView()
        'labelPOV0.Text = pov(0).ToString()
        Dim buttonsState As Byte() = state.GetButtons()
        Dim _button As Integer = 0
        For buttonindex As Integer = 0 To buttonsState.Length - 1
            If IsButtonPressed(buttonsState(buttonindex)) Then
                strText &= String.Format(buttonindex + 1)
            End If
        Next
        'labelButtons.Text = strText
    End Sub


    Private Sub timer1_Tick(ByVal sender As Object, ByVal e As System.EventArgs) Handles timer1.Tick
        'TextBox1.Clear()
        For n = 0 To 23 Step 3
            servo(n) = &H53
        Next
        servo(1) = 1
        servo(4) = 2
        servo(7) = 3
        servo(10) = 4
        servo(13) = 5
        servo(16) = 6
        servo(19) = 7
        servo(22) = 8
        servo(2) = state.X
        servo(5) = state.Y
        servo(8) = state.Z
        servo(11) = state.Rx
        servo(14) = state.Ry
        servo(17) = state.Rz
        servo(20) = state.GetSlider(0)
        servo(23) = 128
        GetData()
        'For n = 0 To 23
        'TextBox1.Text = TextBox1.Text & servo(n).ToString & ","
        'Next
        SerialPort1.Write(servo, 0, 9)

    End Sub





    ''' <summary>
    ''' This subroutine runs when the user clicks the Target=2000us button.
    ''' </summary>
    Sub Button2000_Click(ByVal sender As Object, ByVal e As EventArgs) Handles Button2000.Click
        TrySetTarget(0, 2000 * 4) ' Set the target of channel 0 to 2000 microseconds.
    End Sub

    ''' <summary>
    ''' This function runs when the user clicks the Disable button.
    ''' </summary>
    Sub ButtonDisable_Click(ByVal sender As Object, ByVal e As EventArgs) Handles ButtonDisable.Click
        ' Set target of channel 0 to 0.  This tells the Maestro to stop
        ' transmitting pulses on that channel.  Any servo connected to it
        ' should stop trying to maintain its position.
        TrySetTarget(0, 0)
    End Sub

    ''' <summary>
    ''' Attempts to set the target of 
    ''' </summary>
    ''' <param name="channel">Channel number from 0 to 23.</param>
    ''' <param name="target">
    '''   Target, in units of quarter microseconds.  For typical servos,
    '''   6000 is neutral and the acceptable range is 4000-8000.
    ''' </param>
    Sub TrySetTarget(ByVal channel As Byte, ByVal target As UInt16)
        Try
            Using device As Usc = connectToDevice() ' Find a device and temporarily connect.
                device.setTarget(channel, target)
                ' device.Dispose() is called automatically when the "Using" block ends,
                ' allowing other functions and processes to use the device.
            End Using
        Catch exception As Exception  ' Handle exceptions by displaying them to the user.
            displayException(exception)
        End Try
    End Sub

    ''' <summary>
    ''' Connects to a Maestro using native USB and returns the Usc object
    ''' representing that connection.  When you are done with the
    ''' connection, you should close it using the Dispose() method so that
    ''' other processes or functions can connect to the device later.  The
    ''' "Using" statement can do this automatically for you.
    ''' </summary>
    Function connectToDevice() As Usc
        ' Get a list of all connected devices of this type.
        Dim connectedDevices As List(Of DeviceListItem) = Usc.getConnectedDevices()

        For Each dli As DeviceListItem In connectedDevices
            ' If you have multiple devices connected and want to select a particular
            ' device by serial number, you could simply add some code like this:
            '    If dli.serialNumber <> "00012345" Then
            '        Continue For
            '    End If

            Dim device As Usc = New Usc(dli)  ' Connect to the device.
            Return device                     ' Return the device.
        Next

        Throw New Exception("Could not find device.  Make sure it is plugged in to " & _
            "USB and check your Device Manager.")
    End Function

    ''' <summary>
    ''' Displays an exception (error) to the user by popping up a message box.
    ''' </summary>
    Sub displayException(ByVal exception As Exception)
        Dim stringBuilder As StringBuilder = New StringBuilder()
        Do
            stringBuilder.Append(exception.Message & "  ")
            If TypeOf exception Is Win32Exception Then
                Dim win32Exception As Win32Exception = DirectCast(exception, Win32Exception)
                stringBuilder.Append("Error code 0x" + win32Exception.NativeErrorCode.ToString("x") + ".  ")
            End If
            exception = exception.InnerException
        Loop Until (exception Is Nothing)
        MessageBox.Show(stringBuilder.ToString(), Text, MessageBoxButtons.OK, MessageBoxIcon.Error)
    End Sub


    Private Sub MainWindow_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        If InitDirectInput() = False Then Close()
        SerialPort1.Open()
        timer1.Start()
    End Sub
End Class
